/*****************************************************************************/
/*                                                                           */
/* ad9833.c                                                                  */
/*                                                                           */
/* Routinen zur Ansteuerung des AD9833 DDS-Chips ber ATMEGA8                */
/*                                                                           */
/* (2008) Universitt Stuttgart / IZFM (Dipl.-Ing.(FH) Klaus Skibowski)      */
/*                                                                           */
/*****************************************************************************/

#include <avr/io.h>
#include "ad9833.h"

#define FSYNC_HIGH	PORTB |= 0x04			// PB2=1
#define FSYNC_LOW   PORTB &= ~0x04			// PB2=0

double mclk_faktor;


void ad9833_write(unsigned int wert)
{
  FSYNC_LOW;
  SPDR = (unsigned char)(wert>>8);
	while ( !(SPSR & (1<<SPIF))) {};
  SPDR = (unsigned char)(wert & 0x00FF);
	while ( !(SPSR & (1<<SPIF))) {};
  FSYNC_HIGH;	
}

void ad9833_reset(void)
{
  control_reg |= AD9833_RESET;
  ad9833_write(control_reg);
}

void ad9833_set_frequency(unsigned short reg, double wert)
{
  unsigned int high,low;
	unsigned long freq_word;
	
  freq_word = (unsigned long)(wert*mclk_faktor);
	high = (unsigned int)(((freq_word>>14) & 0x3FFF) | (reg<<14));
	low  =   (unsigned int)((freq_word & 0x00003FFF) | (reg<<14));
	ad9833_write(low);
	ad9833_write(high);
}

void ad9833_set_waveform(unsigned short waveform)
{
  switch(waveform)
	{
	  case WAVEFORM_SINUS:
		{
		  control_reg &= ~AD9833_MODE;  		// Mode=0
			control_reg &= ~AD9833_OPBITEN;		// OPBITEN=0
			ad9833_write(control_reg);
		} break;

	  case WAVEFORM_TRIANGLE:
		{
		  control_reg |= AD9833_MODE;  		  // Mode=1
			control_reg &= ~AD9833_OPBITEN;		// OPBITEN=0
			ad9833_write(control_reg);		
		} break;

	  case WAVEFORM_SQUARE:
		{
		  control_reg &= ~AD9833_MODE;  		// Mode=0
			control_reg |= AD9833_OPBITEN;		// OPBITEN=1
			ad9833_write(control_reg);				
		} break;
	}
}


void ad9833_init(void)
{
  // SPI-Init
  /* Set MOSI and SCK output, all others input */
	DDRB  |= 0x20; // PB5 (SCK)   = Ausgang 
	DDRB  |= 0x08; // PB3 (MOSI)  = Ausgang 
	DDRB  |= 0x04; // PB2 (FSYNC) = Ausgang 
	FSYNC_HIGH;

  // SPE=1, MSTR=1, f_clk = 4 MHz
  SPCR = (1<<SPE)|(1<<MSTR)|(1<<CPOL);	
	
	mclk_faktor = 268435456/(double)MCLK;
	
  waveform=WAVEFORM_SINUS;
	wobbel_mode = 0;
	control_reg=AD9833_B28 | AD9833_RESET | AD9833_DIV2;
}

